#ifndef __include_appconf_h__
#define __include_appconf_h__

#include <stdarg.h>

enum conf_type {
	conf_type_any,

	conf_type_bool,
	conf_type_int,
	conf_type_float,
	conf_type_str,
	conf_type_range,
	conf_type_enum,
	conf_type_strarray,

	conf_type_group,
};
struct conf_valid_range_data {
	int lower_limit, upper_limit;
};
struct conf_value;
struct conf_enum_list;
struct conf_group;

typedef int (*conf_foreach_func)(struct conf_group *g, const char *name,
		struct conf_value *v, void *d);

struct conf_enum_list {
	int num;
	const char *str;
};
#define conf_free_val(cv)	do { } while(0)	

const char *conf_value_to_str_const(struct conf_value *val);
int conf_value_to_bool(struct conf_value *val, int *b);
int conf_value_to_str(struct conf_value *val, char *buffer, size_t size);
int conf_value_to_int(struct conf_value *val, int *i);
int conf_value_to_float(struct conf_value *val, double *f);
int conf_value_to_enum(struct conf_value *val, int *v, 
		struct conf_enum_list *el);
int conf_value_to_strarray(POOL p,struct conf_value *val, char ***a);

struct conf_group *conf_value_to_group(struct conf_value *val);




struct conf_value *conf_value_to_type(POOL p,struct conf_value *cv, 
		enum conf_type type);

const char *conf_group_name(struct conf_group *cg);



const char *conf_type_to_str(enum conf_type type);




int conf_value_foreach(struct conf_group *group, conf_foreach_func func);
int conf_foreach(struct conf_group *cg, conf_foreach_func func, void *appdata);




struct conf_value *conf_get_value(POOL p, const char *valuepath, ...);	
struct conf_value *conf_get_value_v(POOL p, const char *valuepath, va_list vl);	
struct conf_value *conf_get_value_str(POOL p, const char *valuepath);	
/* Returns the conf_value subsetting in group cg named name */
struct conf_value *conf_group_value(POOL p, struct conf_group *cg, 
		const char *name);
/* Returns the nth conf_value subsetting in group cg */
struct conf_value *conf_group_value_n(POOL p, struct conf_group *cg, 
		int num);





struct conf_value *conf_new_enum(POOL p, int i);
struct conf_value *conf_new_int(POOL p, int i);
struct conf_value *conf_new_str(POOL p, const char *str);
struct conf_value *conf_new_bool(POOL p, int i);
struct conf_value *conf_new_float(POOL p, double f);
struct conf_value *conf_new_group_val(POOL p, struct conf_group *cg);
struct conf_value *conf_new_strarray(POOL p, char **a);

int appconf_load_config(const char *filename);
int appconf_loadfile(const char *filename);

void conf_missing(DEBUG h,const char *path);
void conf_invalid(DEBUG h,const char *path, enum conf_type type);
void conf_debug_strarray(DEBUG h, char **strarray);

int parse_enum_list(const char *str,struct conf_enum_list *el, int *result);
#endif
